/***************************************************************************************
 * @file     I2C_AN_main.c
 * @version  V1.0.9
 * @date     3/12/2019  CEM 
 *
 * @note
 * VORAGO Technologies 
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies. 
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY 
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT. 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES 
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
/****************************************************************************************/
/* Include Files                                                                        */
/****************************************************************************************/
#include <stdio.h>
#include "va108xx.h"
#include "reb_log.h"
#include "reb_board.h"
#include "reb_adt75.h"
#include "driver_common.h"
#include "reb_max11619.h"
#include "reb_timer.h"
#include "gpio_va108xx.h"
#include "segger_rtt.h" 
#include "irq_va108xx.h"

/****************************************************************************************/
/* local pre-processor symbols/macros ('#define')                                                       */
/****************************************************************************************/

#define SEGGER_RTT_IN_RAM 1

#define I2CB_INTn  22 
#define I2CB_Priority  1  

#define PORTB13  13   
	 
#define ADC_I2C_ADDRESS 0x9A 
#define TEMP_SEN_I2C_ADDRESS 0x92 
#define DAC_I2C_ADDRESS 0x90
#define DAC_default_out 0xf0 			

#define I2CB_CLKSCALE_VAL 0x0018    // default to normal mode, divide by 25 for 50 MHz SysClk 

#define I2C_START_STOP_CMD 0x03  ;  // start with stop 

#define PERIPH_CLK_ENAB_PORTA 0x00 
#define PERIPH_CLK_ENAB_PORTB 0x01
#define PERIPH_CLK_ENAB_SPIA 0x04
#define PERIPH_CLK_ENAB_SPIB 0x05
#define PERIPH_CLK_ENAB_SPIC 0x06
#define PERIPH_CLK_ENAB_UARTA 0x8
#define PERIPH_CLK_ENAB_UARTB 0x9
#define PERIPH_CLK_ENAB_I2CA 0x10
#define PERIPH_CLK_ENAB_I2CB 0x11
#define PERIPH_CLK_ENAB_IRQSEL 0x15
#define PERIPH_CLK_ENAB_IOCONFIG 0x16
#define PERIPH_CLK_ENAB_UTILITY 0x17
#define PERIPH_CLK_ENAB_PORTIO 0x18 
#define FUNCSEL0 00
#define FUNCSEL1 01 
#define FUNCSEL2 02 

/****************************************************************************************/
/* Global variables declared here                                                       */
/****************************************************************************************/

volatile uint8_t bounce_cnt = 0; 

volatile uint32_t ADC_cnt,  ADC_results[5] ;   //  used inside ISR and in main routine  
char abDataIn[128], abDataOut[128] ;   //  RTT (real time terminal) buffer allocation
 
/*****************************************************************************/ 
/* Local type definitions ('typedef')                                        */ 
/*****************************************************************************/
// none 
/*****************************************************************************/ 
/* Local variable definitions ('static')                                     */ 
/*****************************************************************************/
// none 
/*****************************************************************************/ 
/* Local function prototypes ('static')                                      */ 
/*****************************************************************************/
// none 

// *********************

uint32_t  __attribute__((aligned(32))) READ_APB(volatile const uint32_t *ptr) {
	 return *ptr;
}

void  __attribute__((aligned(32))) WRITE_APB(volatile uint32_t *ptr, uint32_t data) {
	 *ptr=data;
}


//************ init_i2cb *********
//  Steps: 
//     0: Enable clocks to I2CB
//     1: Load CLKSCALE to set bit rate  
//     2: Clear Rx and Tx FIFO
//     3: Load CTRL with parameters (dig filter enabled & enable bit) 

uint32_t init_i2cb() 
{
  VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_I2CB  ;  
	
	VOR_I2CB -> CLKSCALE = I2CB_CLKSCALE_VAL  ;  // default to normal mode, divide by 25 for 50 MHz SysClk  
	VOR_I2CB -> FIFO_CLR =(I2CB_FIFO_CLR_RXFIFO_Msk | I2CB_FIFO_CLR_TXFIFO_Msk) ;// clear both Rx and Tx FIFO 
	VOR_I2CB -> CTRL =  (I2CB_CTRL_DLGFILTER_Msk | I2CB_CTRL_ENABLE_Msk)  ;   // enable filter and module 
	return(VOR_I2CB -> STATUS)  ; 
}

//  ****** Write to DAC on I2CB     *********     
// 
//   Basic flow is this:
//      1) init clocks for peripherals
//			2) init SPI control registers (0 & 1)  
//      3) send Write enable command (single byte)  {wait for transmit complete} 
//      4) wait for frame to complete

uint32_t write_DAC_I2CB (uint32_t I2C_Address, uint32_t DAC_out) 
{
volatile		 int32_t count = 0, count2=0, result, temp0, temp1, temp_ctrl, xtemp, x  ; 
	
	VOR_I2CB -> WORDS = 3  ;   // DAC has address + two bytes of data
	VOR_I2CB -> ADDRESS = I2C_Address & ~ 0x01  ;  // lsb = 0 (write) 
	VOR_I2CB -> DATA = 0x0 ;   // Address = 0x0 for writing data
	VOR_I2CB -> DATA = (DAC_out >> 2) & ~0xFFFFF00  ;  //  write first 8 bits of DAC_out (upper 8 bits of 10)
	VOR_I2CB -> DATA  = (DAC_out << 6) & ~0xFFFF3F  ;  // write 2nd byte of output data (lower 2 bits in pos 7 & 6)
	VOR_I2CB -> CMD = I2C_START_STOP_CMD  ;  // start with stop 
	
	while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0)  // wait here until I2CB is idle 
	{
		count++ ;  
		if (count > 0x100000) 
		break   ; 
	}
	return(VOR_I2CB -> STATUS)  ; 
}

//  ****** Read ADC on I2CB     *********
// Steps:
//  1) Load WORDS with size of transaction 
//  2) Load I2C ADDRESS of ADC (provided in Subroutine call) 
//  4) Load CMD register with 0x03 (start with stop) 
//  5) Poll STATUS until transaction is complete
//  6) Process Rx_FIFO, shift and concatentate data for 10-bit result 

uint32_t read_adc_i2cb (uint32_t I2C_Address)
{
	volatile		 int32_t count = 0, result ;  
	
	VOR_I2CB -> WORDS = 2  ;   // ADC has two byte result
	VOR_I2CB -> ADDRESS = I2C_Address | 0x01  ;  // set slave address for read (lsb =1)
	VOR_I2CB -> CMD = I2C_START_STOP_CMD  ;  // start with stop 
	
	while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0) // wait here until I2CB is idle 
	{
		count++ ;  
		if (count > 0x100000) 
		break   ; 
	}

  result = VOR_I2CB -> DATA ; 
  result = (result << 0x08) | VOR_I2CB -> DATA ;  // shift over first byte, read second byte and combine the two
	
	return(result)  ; 
} 
		

//  ****** Read Temperature Sensor on I2CB  *********
// Steps:
//  1) Write address to Read temp register 
// 		1a)  Load WORDS with size of transaction = 1 byte address  
// 		1b)) Load I2C ADDRESS of Temp sensor = 0x92 
//    1c) Load required DATA in to DATA FIFO (address for temp info = 0x0)  
//    1d) Load CMD register with 0x03 (start with stop) 
//    1e) Poll STATUS until transaction is complete
//  2) Read temp sensor information 
// 		2a) Load WORDS with size of transaction =2 byte address  
// 		2b) Load I2C ADDRESS of Temp sensor = 0x92 
//    2c) Load CMD register with 0x03 (start with stop) 
//    2e) Poll STATUS until transaction is complete
//    2f) Read RxFIFO and align result

uint32_t read_temp_i2cb ( uint32_t I2C_Address )

{
volatile		 int32_t count = 0, count2=0, result, temp0, temp1 ;  
	
	VOR_I2CB -> WORDS = 1  ;   // Temp sensor has one byte result
	VOR_I2CB -> ADDRESS = I2C_Address & ~ 0x01  ;   // set lsb = 0 (write) 
  VOR_I2CB -> DATA = 0x0 ;   // Address = 0x0 for writing data	
	VOR_I2CB -> CMD = I2C_START_STOP_CMD  ;  // start with stop 
	
	while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0) // wait for transaction to complete
	{
		count++ ;  
		if (count > 0x100000) 
		break   ; 
	}

	VOR_I2CB -> WORDS = 2  ;   // Temp sensor has one byte result
	VOR_I2CB -> ADDRESS = I2C_Address | 0x01  ;  // set slave address with lsb =1 (read) 
	VOR_I2CB -> CMD = I2C_START_STOP_CMD  ;  // start with stop 

	while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0) // wait for transaction to complete
	{
		count++ ;  
		if (count > 0x100000) 
			break   ; 
	}
  temp0 = VOR_I2CB -> DATA ; 
	temp1 = VOR_I2CB -> DATA ; 
	result = (temp0 << 0x08) | temp1   ;  // combine two bytes to make single result 
	return(result)  ; 
} 

//*************************************************************************
	

//  ****** Initialize I2CB for interrupts on IRQ22     *********
// Steps:
//  1) Configure I2CB
//  2) Configure IRQ_SEL peripheral 
//  3) Setup NVIC and enable interrupts on IRQ22 
// 	

int32_t init_i2cb_IRQ22()  
{
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_I2CB  ;  
	
	VOR_I2CB -> CLKSCALE = I2CB_CLKSCALE_VAL  ;  // default to normal mode, divide by 25 for 50 MHz SysClk
	VOR_I2CB -> FIFO_CLR =(I2CB_FIFO_CLR_RXFIFO_Msk | I2CB_FIFO_CLR_TXFIFO_Msk) ;// clear both Rx and Tx FIFO 
	VOR_I2CB -> CTRL =  (I2CB_CTRL_DLGFILTER_Msk | I2CB_CTRL_ENABLE_Msk)  ;   // enable filter and module 
  VOR_I2CB -> IRQ_ENB = I2CB_STATUS_IDLE_Msk  ;  // generate interrupt when IDLE = 1 

  VOR_IRQSEL->I2C_MS[1] = I2CB_INTn   ;    

  NVIC_SetPriority(I2CB_INTn,I2CB_Priority)  ;  // set priority in NVIC 
  NVIC_EnableIRQ(I2CB_INTn)  ;   // enable IRQ for I2CB

  return(VOR_I2CB -> STATUS  )  ;  	
}

//  ******* Interrupt Subroutine for I2CB  (OC22_IRQHandler)  
//  This will kick off a series of 4, 3-byte i2cb transactions to read a 10-bit ADC
//   After the 4th read, the I2CB will be idled and interrupts disabled 
//  The first time into the interrupt will be for the idle status prior to any i2c transactions 
// Entries 2-5 will be for idle condition post ADC conversion
// ADC_cnt is global variable which is set to zero before INT22 is enabled 
//  Individual ADC results are stored in global variables:  ADC_result[ADC_cnt]  

void OC22_IRQHandler (void)  
{
	uint32_t result, count  ;  
	
	VOR_GPIO->BANK[1].DATAOUT |= (1<<PORTB13)  ;  
	
	if(ADC_cnt > 0)  // process data in RX FIFO for entries 2-5  (ADC_cnt:  1-4 )  
	{
		result = VOR_I2CB -> DATA  ;  
		result = (result << 0x08) | VOR_I2CB -> DATA  ;  // shift over first 
		ADC_results[ADC_cnt] = result  ;  
  } 
  
	if(ADC_cnt > 3)  // this was fourth sample so shut down I2CB)  
	{
	  VOR_I2CB -> IRQ_ENB = 0x00  ;  // generate interrupt when IDLE = 0 
		ADC_results[0] = (ADC_results[1] + ADC_results[2] +ADC_results[3]+ ADC_results[4]) / 4  ; 
	}	 
  else
	{
    VOR_I2CB -> WORDS = 2  ;   // Temp sensor has one byte result
	  VOR_I2CB -> ADDRESS = 0x9A | 0x01  ;  // set slave address with lsb =1 (read) 
	  VOR_I2CB -> CMD = I2C_START_STOP_CMD  ;  // start with stop     

	
	  while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) != 0) // wait for transaction to start 
	   {
		  count++ ;  
		  if (count > 0x100000) 
				break   ; 
	   }
		VOR_I2CB -> IRQ_CLR = 0xFFFF  ;  // clear all pending interrupts 
		VOR_I2CB -> IRQ_ENB = I2CB_STATUS_IDLE_Msk  ;  // generate interrupt when IDLE = 1  
	}	

  ADC_cnt ++ ;  //  increment counter 	
	VOR_GPIO->BANK[1].DATAOUT &= ~(1<<PORTB13)  ;  //  debug use only clear pin on ISR exit 
	
}

//**************************************************************

//------------------------------------------------------------------------------
// Main Routine 
//  Run through each example serially then loop on DAC out, ADC read and Temp Sensor read 
//  Example 1:  DAC out 
//  Example 2:  ADC and Temp sensor reads 
//  Example 3:  Interrupt driven I2C read of ADC (4 reads + averaging)  
//------------------------------------------------------------------------------

int main(void) 
{
volatile int32_t i =0, kk, adc_temp[16], temp_temp[16]; 
volatile int32_t systick_old, elapsed_ticks1, main_cnt, delay_cnt = 0; 

	
  VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= 
	       ((1<<PERIPH_CLK_ENAB_PORTIO) | (1<<PERIPH_CLK_ENAB_IOCONFIG) 
	        | (1<<PERIPH_CLK_ENAB_PORTA) | (1<<PERIPH_CLK_ENAB_PORTB)
	        | (1<<PERIPH_CLK_ENAB_IRQSEL) |(1<<PERIPH_CLK_ENAB_SPIA)); // enable clks to PORTIO, IOMGR and SPIB peripherals 
	
	for(i=0; i<32; i++  )   //  config all port pins with internal pull resistor enabled 
	{
	  VOR_IOCONFIG->PORTA[i] |= IOCONFIG_PORTA_PEN_Msk  ;
	  if(i<24) 
		{ 
		  VOR_IOCONFIG->PORTB[i] |= IOCONFIG_PORTB_PEN_Msk  ;
		}
	}
  VOR_GPIO->BANK[0].DIR |= ( (0x01UL << PORTA_6_D4) |(0x01UL << PORTA_7_D3) | (0x01UL << PORTA_10_D2) ) ; // setup REB1 LED pins  : PORTA 6,7,10
	VOR_GPIO->BANK[0].DATAOUT &= ~((0x01UL << PORTA_6_D4) |(0x01UL << PORTA_7_D3)  )  ; //drive  2 LEDs off
	VOR_GPIO->BANK[0].DATAOUT |= (0x01UL << PORTA_10_D2)   ; //drive 1 LED on 

	SysTick -> CTRL = (SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk) ; /*  enable Systick counter  */
	SysTick -> LOAD =  0x7fffff ; 
  systick_old = SysTick ->VAL ; 
		
	SEGGER_RTT_Init() ; //  Must call this for RAM based apps 
	SEGGER_RTT_ConfigUpBuffer(1, "DataOut", &abDataOut, sizeof(abDataOut), SEGGER_RTT_MODE_BLOCK_IF_FIFO_FULL);
  SEGGER_RTT_ConfigDownBuffer(1, "DataIn", &abDataIn[0], sizeof(abDataIn), SEGGER_RTT_MODE_NO_BLOCK_SKIP);
 
	SEGGER_RTT_printf(0, "\n\n*****  AN1208 Example I2C code terminal output  *****\n\n")  ; 	

// **  Example 1 - DAC write of AN1208 starts here 
	SEGGER_RTT_printf(0, "\n\n>>>>>  Example 1 - Write to DAC - polling method \n")  ; 
		
	init_i2cb()  ;  
	write_DAC_I2CB(DAC_I2C_ADDRESS,DAC_default_out)   ;  
	SEGGER_RTT_printf(0, "\nI2C DAC ADD : %5X", DAC_I2C_ADDRESS  )  ;
	SEGGER_RTT_printf(0, "\nI2C DAC OUT : %5X", DAC_default_out  )  ;
    
// **  Example 2 - Read ADC & Temp Sensor of AN1208 starts here   **  
  SEGGER_RTT_printf(0, "\n\n>>>>>  Example 2 - ADC & Temp Sensor read - polling method \n")  ; 

  init_i2cb()  ;  
  adc_temp[0] = read_adc_i2cb(ADC_I2C_ADDRESS)   ;  
		
  SEGGER_RTT_printf(0, "\n\nI2C ADC ADD : %5X", ADC_I2C_ADDRESS  )  ;
	SEGGER_RTT_printf(0, "\nI2C ADC result : %5X", adc_temp[0]  )  ;
		
	init_i2cb()  ; 
	temp_temp[0] = read_temp_i2cb(TEMP_SEN_I2C_ADDRESS)  ;  
		
	SEGGER_RTT_printf(0, "\n\nI2C Temp Sensor ADD %5X", TEMP_SEN_I2C_ADDRESS  )  ;
	SEGGER_RTT_printf(0, "\n\nI2C Temp Sensor result %5X", temp_temp[0]  )  ;	
				
// ** Example 3 - ADC read with Interrupts enabled of AN1208 starts here  **  
		SEGGER_RTT_printf(0, "\n\n>>>>> Example 3 - ADC read with interrupt enabled\n")  ; 			
	  
		init_i2cb_IRQ22() ;   // data from ADC read with Interrupt enabled 
		    
		kk = 1 ; 
		  
		while(1)     //  This is infinite loop  (reads ADC and Temp Sensor on I2CB continuously) 
		{
	    systick_old = SysTick ->VAL ;   //  start the stopwatch 
		  kk ++ ;  // kk is used as pointer for 
			if (kk > 15) 
      { 
			  kk = 1; 
			}  			 
			 
		 VOR_GPIO->BANK[0].TOGOUT = (0x01UL << 6) ; // toggle LEDs each time through 
			 
		 if(ADC_cnt >= 5)     //  Must wait for interrupt driven transactions to complete 
		 {
		   init_i2cb()  ;  
		   adc_temp[kk] = read_adc_i2cb(ADC_I2C_ADDRESS)   ;  
			
		   SEGGER_RTT_printf(0, "I2C ADC ADD    : %5X \n", ADC_I2C_ADDRESS )  ;
		   SEGGER_RTT_printf(0, "I2C ADC result : %5X Count(1-F) : %5X \n", adc_temp[kk], kk )  ;
		
		   init_i2cb()  ; 
		   temp_temp[kk] = read_temp_i2cb(TEMP_SEN_I2C_ADDRESS)  ;  
		
		   SEGGER_RTT_printf(0, "I2C Temp Sensor ADD    :  %5X\n", TEMP_SEN_I2C_ADDRESS  )  ;
		   SEGGER_RTT_printf(0, "I2C Temp Sensor result :  %5X  Count(1-F) %5X\n", temp_temp[kk], kk )  ;	
		
     } 
		 
		 elapsed_ticks1 = systick_old - SysTick ->VAL   ;  // measure the clock cycles to perform I2CB transactions 
		 SEGGER_RTT_printf(0, "Cycle count for I2CB read of ADC and Temp Sensor %8X\n\n", elapsed_ticks1  )  ;
	 
		 for(delay_cnt=0;delay_cnt < 1000000; delay_cnt++) {  ; }   // primitive delay to keep RTT buffer from overflowing 
		
		 main_cnt ++ ; 
		}  //  end of while (1)   
		
				
	}  // end of main routine 
	
	
 
	





